<?php
// $Id: taxonomy_vtn.pages.inc,v 1.1.2.10 2008/07/01 09:19:17 tomaszx Exp $

/**
 * @file
 * pages for display vocabularies, terms and nodes
 *
 */


/**
 * Implementation of taxonomy_vtn_show_vocabularies().
 */
function taxonomy_vtn_show_vocabularies() {

  $vocs = taxonomy_get_vocabularies();

  // exclude some vocs_id
  $omit_vocs = explode(', ', variable_get('taxonomy_vtn_vocabularies_omit', ''));

  //$omit_vocs = array_filter(variable_get('taxonomy_vtn_vocabularies_omit', array()));
  if (!empty($omit_vocs)) {
    foreach ($omit_vocs as $omit_vocs_id) {
      unset($vocs[$omit_vocs_id]);
    }
  }

  if ( (count($vocs) == 1) && (variable_get('taxonomy_vtn_vocabularies_goto_terms_if_one', 0) == 1) ) {
    drupal_goto('taxonomy_vtn/voc/'. key($vocs));
  }

  //Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_vocs');
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    return $output;
  }
  else {

    if ($vocs) {

      $show_empty_vocs = variable_get('taxonomy_vtn_vocabularies_show_empty_vocs', 1);
      $show_count_terms = variable_get('taxonomy_vtn_vocabularies_show_count_terms', 1);
      $show_voc_desc = variable_get('taxonomy_vtn_vocabularies_show_voc_desc', 1);
      $count_group_vocabularies =  variable_get('taxonomy_vtn_vocabularies_count_group_vocabularies', 1);
      $count_column_vocs =  variable_get('taxonomy_vtn_vocabularies_count_column_vocs', 2);
      $arr = array();
      $is_desc = false;

      foreach ( $vocs as $vid => $voc_obj ) {
        // TODO: this not exclude no published
        $terms = taxonomy_get_tree($vid);
        $tcount = count($terms);

        if ($show_count_terms) {
          $count_terms = ' ('. $tcount .')';
        }

        if ($show_voc_desc) {
          $voc_desc = '<br /><span class="description">'. check_plain($voc_obj->description) .'</span>';
          $is_desc = true;
        }

        if ($tcount || $show_empty_vocs) {
          $index = drupal_strtoupper(drupal_substr($voc_obj->name, 0, $count_group_vocabularies));
          $arr[$index][] = l($voc_obj->name, 'taxonomy_vtn/voc/'. $voc_obj->vid) . $count_terms . $voc_desc;
        }
      }


      $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');

      // change locale to sort
      if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
        setlocale(LC_ALL, $my_locale);
        ksort($arr, SORT_LOCALE_STRING);
        // set locale back to default
        setlocale(LC_ALL, NULL); // without this count($arr) not work properly
      }
      else {
        ksort($arr); //--- this not support locales
      }


      $cnt = ceil(count($arr)/$count_column_vocs);
      $k = 0; $col = 0;
      foreach ($arr as $i => $t) {
        if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
          $cell = 'cell'. ++$col;
        }
        $$cell .= theme('item_list', $t, $i, 'ul');
        ++$k;
      }

      if ($k) {
        // prepare cells and they width
        ($col < $count_column_vocs ) ? $ile = $col : $ile = $count_column_vocs;
        for ($num=1; $num <= $ile; ++$num) {
          $xcell = 'cell'. $num;
          $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
        }
        $row[] = $xcells;


        if ($show_voc_desc && $is_desc) {
          $output .= '<div class="taxonomy_vtn-navigation">';
          $output .= '<a  class="show-hide-desc" href="javascript:taxonomy_vtn_show_hide_desc(\'vocabularies\');">'. t('Show / Hide descriptions') .'</a>';
          $output .= '</div>';
        }

        $output .= theme('table', array(), $row, array('class' => 'taxonomy_vtn_vocabularies', 'id' => 'taxonomy_vtn_vocabularies'));

        //Cache the fully rendered page in case of aggressive caching
        if (variable_get('taxonomy_vtn_caching', 0) == 1) {
          cache_set('taxonomy_vtn_show_vocs', $output);
        }

        return $output;
      }
      else {

        $output = '<p class="no-data-info">';
        $output .= t('No data to display. Probably you have set option to not show empty vocabularies.');
        $output .= '</p>';

        //Cache the fully rendered page in case of aggressive caching
        if (variable_get('taxonomy_vtn_caching', 0) == 1) {
          cache_set('taxonomy_vtn_show_vocs', $output);
        }
        return $output;
      }
    }
    else {
      $output  = '<p class="no-data-info">';
      $output .= t('No data to display. Probably you not have any vocabularies');
      $output .= '</p>';

      //Cache the fully rendered page in case of aggressive caching
      if (variable_get('taxonomy_vtn_caching', 0) == 1) {
        cache_set('taxonomy_vtn_show_vocs', $output);
      }

      return $output;
    }
  } // end cache
}

/**
 * Implementation of taxonomy_vtn_show_terms().
 */
function taxonomy_vtn_show_terms($arg = '') {

  // check for excluded voc_id
  $omit_vocs = explode(', ', variable_get('taxonomy_vtn_vocabularies_omit', ''));

  if (!empty($omit_vocs)) {
    if (in_array($arg, $omit_vocs)) {
      $arg = null;
    }
  }

  if (is_numeric($arg)) {
    $own_vid = (int)$arg;
    if ($voc_obj = taxonomy_vocabulary_load($own_vid)) {
      $title = '';
      if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 0) {
        $title = t('Terms in') .' ';
      }
      drupal_set_title($title . t('@vocabulary', array('@vocabulary' => $voc_obj->name)));
      $output = '<div class="description">'. check_plain($voc_obj->description) .'</div>';
    }
    else {
      drupal_set_title(t('Bad vocabulary id'));
      $output  = '<p class="no-data-info">';
      $output .= t('No data to display. Probably you not have a vocabulary with vid = !voc_id', array('!voc_id' => $own_vid));
      $output .= '</p>';
      $terms_navigation = '';
      if (variable_get('taxonomy_vtn_terms_show_back_to_voc', 1) == 1) {
        $where_back = t('vocabularies');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
          $where_back = t('master index');
        }
        $terms_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
        $output .= '<div class="taxonomy_vtn-navigation">'. $terms_navigation .'</div>';
      }
      return $output;
    }
  }
  else {
    drupal_set_message(t('Please select a vocabulary first'));
    drupal_goto('taxonomy_vtn');
  }


  //Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_terms_'. $own_vid);
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    return $output;
  }
  else {

    // vocabulary vid is ok, lets do it

    $show_empty_terms = variable_get('taxonomy_vtn_terms_show_empty_terms', 1);
    $show_count_nodes = variable_get('taxonomy_vtn_terms_show_count_nodes', 1);
    $show_term_desc = variable_get('taxonomy_vtn_terms_show_term_desc', 1);
    $show_synonyms = variable_get('taxonomy_vtn_terms_show_synonyms', 1);
    $count_group_terms =  variable_get('taxonomy_vtn_terms_count_group_terms', 1);
    $count_column_terms =  variable_get('taxonomy_vtn_terms_count_column_terms', 2);
    $term_goto = variable_get('taxonomy_vtn_terms_goto', 'nodes');
    $show_back_to_voc = variable_get('taxonomy_vtn_terms_show_back_to_voc', 1);

    // TODO: for depth use:  if ( $terms = taxonomy_get_tree($own_vid,0,-1,3) ) {
    if ( $terms = taxonomy_get_tree($own_vid) ) {
      $arr = array();
      $arr_terms = array();
      $row = array();
      $count_nodes = '';
      $is_desc = false;

      // exclude some terms_id
      $omit_terms = explode(', ', variable_get('taxonomy_vtn_terms_omit', ''));

       /// prepare own full structure with terms and synonyms and link to them
      foreach ($terms as $term) {
          //$cnt_nodes[$term->tid] = taxonomy_term_count_nodes($term->tid);

        // exclude terms
        if (!in_array($term->tid, $omit_terms)) {

          switch ($term_goto) {
              case 'nodes':
                  $term_goto_path = 'taxonomy_vtn/term/'. $term->tid;
                  $tcount = taxonomy_term_count_nodes($term->tid);
                  break;

              case 'taxonomy_term':
                  $term_goto_path = 'taxonomy/term/'. $term->tid;
                  // real count
                  $tcount = db_result(db_query("SELECT COUNT(nid) FROM {term_node} WHERE tid = %d", $term->tid));
                  break;
              case 'taxonomy_term_all':
                  $term_goto_path = 'taxonomy/term/'. $term->tid .'/all';
                  $tcount = taxonomy_term_count_nodes($term->tid);
                  break;
          }

          if ($show_count_nodes) {
            $count_nodes = ' ('. $tcount .')';
          }

          $term_desc = '';
          if ($show_term_desc && $term->description) {
            $term_desc = '<br /><span class="description">'. check_plain($term->description) .'</span>';
            $is_desc = true;
          }

          if ($tcount || $show_empty_terms) {
            $arr_terms[$term->name]['term'][] = l($term->name, $term_goto_path) . $count_nodes . $term_desc ;
            // if is not empty term or we show empty terms
            // and we will show synonyms then get for this
            if ($show_synonyms) {
              $synonyms = taxonomy_get_synonyms($term->tid);
              foreach ($synonyms as $synonym_name) {
                // $synonym_term = taxonomy_get_synonym_root($synonym_name);
                // all synonyms have a target - this term
                $arr_terms[$synonym_name]['synonym'][] = l($term->name, $term_goto_path);
              }
            }
          }
        } // exclude end
      } // end foreach terms as term prepare


      $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');

      // change locale to sort
      if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1 ) {
        setlocale(LC_ALL, $my_locale);
        ksort($arr_terms, SORT_LOCALE_STRING);
        // set locale back to default
        setlocale(LC_ALL, NULL); // without this count($arr) not work properly
      }
      else {
        ksort($arr_terms); //--- this not support languages
      }

      // Now we make index with links to proper terms for synonyms
      foreach ($arr_terms as $name => $types) {
        $index = drupal_strtoupper(drupal_substr($name, 0, $count_group_terms));
        foreach ($types as $type => $links) {
            if ($type == 'term') {
              foreach ($links as $link) { // maybe is more than one
                $arr[$index][] = $link;
              }
            }
            else { // is a synonym then look for terms
              $x_links = array();
              foreach ($links as $link) { // propably is more than one
                $x_links[] = $link;
              }
              $arr[$index][] = $name .'<br /> <small>'. t('Synonyms') .': '. implode(', ', $x_links) .'</small>';
            }
        }
      } // end foreach arr_terms


      // sort array key (index like A B C .. etc.)
      // change locale to sort
      if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
        setlocale(LC_ALL, $my_locale);
        ksort($arr, SORT_LOCALE_STRING);
        // set locale back to default
        setlocale(LC_ALL, NULL); // without this count($arr) not work properly
      }
      else {
        ksort($arr); //--- this not support languages
      }

      $cnt = ceil(count($arr)/$count_column_terms);
      $k = 0; $col = 0;
      foreach ($arr as $i => $t) {
        if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
          $cell = 'cell'. ++$col;
        }
        $$cell .= theme('item_list', $t, $i, 'ul');
        ++$k;
      }

      if ($k) {
        // prepare cells and they width
        ($col < $count_column_terms ) ? $ile = $col : $ile = $count_column_terms;
        for ($num=1; $num <= $ile; ++$num) {
          $xcell = 'cell'. $num;
          $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
        }
        $row[] = $xcells;

        $terms_navigation = array();
        if ($show_back_to_voc) {
          $where_back = t('vocabularies');
          if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
            $where_back = t('master index');
          }
          $terms_navigation[] = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
        }

        if ($show_term_desc && $is_desc) {
          $terms_navigation[] = '<a class="show-hide-desc" href="javascript:taxonomy_vtn_show_hide_desc(\'terms\');">'. t('Show / Hide descriptions') .'</a>';
        }

        if ($show_back_to_voc || $show_term_desc) {
          $output .= '<div class="taxonomy_vtn-navigation">'. implode(' &nbsp; | &nbsp; ', $terms_navigation) .'</div>';
        }

        $output .= theme('table', array(), $row, array('class' => 'taxonomy_vtn_terms', 'id' => 'taxonomy_vtn_terms'));

        //Cache the fully rendered page in case of aggressive caching
        if (variable_get('taxonomy_vtn_caching', 0) == 1) {
          cache_set('taxonomy_vtn_show_terms_'. $own_vid, $output);
        }

        return $output;
      }
      else {
        $output .= '<p class="no-data-info">';
        $output .= t('No data to display. Probably you have set option to not show empty terms.');
        $output .= '</p>';
        $terms_navigation = '';
        if ($show_back_to_voc) {
          $where_back = t('vocabularies');
          if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
            $where_back = t('master index');
          }
          $terms_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
          $output .= '<div class="taxonomy_vtn-navigation">'. $terms_navigation .'</div>';
        }

        //Cache the fully rendered page in case of aggressive caching
        if (variable_get('taxonomy_vtn_caching', 0) == 1) {
          cache_set('taxonomy_vtn_show_terms_'. $own_vid, $output);
        }

        return $output;
      }
    }// end if terms
    else {
      $output .= '<p class="no-data-info">';
      $output .= t('No data to display. Probably this vocabulary is empty.');
      $output .= '</p>';
      $terms_navigation = '';
      if ($show_back_to_voc) {
        $where_back = t('vocabularies');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
          $where_back = t('master index');
        }
        $terms_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
        $output .= '<div class="taxonomy_vtn-navigation">'. $terms_navigation .'</div>';
      }

      //Cache the fully rendered page in case of aggressive caching
      if (variable_get('taxonomy_vtn_caching', 0) == 1) {
        cache_set('taxonomy_vtn_show_terms_'. $own_vid, $output);
      }

      return $output;
    }

  } // end else if cache

}


/**
 * Implementation of taxonomy_vtn_show_nodes().
 */
function taxonomy_vtn_show_nodes($arg = '') {

  // check for excluded voc_id
  $omit_terms = explode(', ', variable_get('taxonomy_vtn_terms_omit', ''));

  if (!empty($omit_terms)) {
    if (in_array($arg, $omit_terms)) {
      $arg = null;
    }
  }

  if (is_numeric($arg)) {
    $own_tid = (int)$arg;
    if ($term_obj = taxonomy_get_term($own_tid)) {
      $title = '';
      if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 0) {
        $title = t('Nodes of term') .' ';
      }
      drupal_set_title($title . t('@term_name', array('@term_name' => $term_obj->name)));
    }
    else {
      drupal_set_title(t('Bad term id'));
      $output  = '<p class="no-data-info">';
      $output .= t('No data to display. Probably you not have a term with tid = !term_id', array('!term_id' => $own_tid));
      $output .= '</p>';

      $terms_navigation = '';
      if (variable_get('taxonomy_vtn_terms_show_back_to_voc', 1) == 1) {
        $where_back = t('vocabularies');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
          $where_back = t('master index');
        }
        $terms_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
        $output .= '<div class="taxonomy_vtn-navigation">'. $terms_navigation .'</div>';
      }
      return $output;
    }
  }
  else {
    drupal_set_message(t('Please select a vocabulary first'));
    drupal_goto('taxonomy_vtn');
  }

  //Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_nodes_'. $own_tid);
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    return $output;
  }
  else {

    $show_count_comments = variable_get('taxonomy_vtn_nodes_show_count_comments', 1);
    $count_column_nodes = variable_get('taxonomy_vtn_nodes_count_column_nodes', 2);
    $show_empty_count_comments = variable_get('taxonomy_vtn_nodes_show_empty_count_comments', 0);
    $show_nodes_index = variable_get('taxonomy_vtn_nodes_show_nodes_index', 0);
    $show_back_to_terms = variable_get('taxonomy_vtn_nodes_show_back_to_terms', 1);
    
    if ($use_pager = (bool)variable_get('taxonomy_vtn_nodes_use_pager', FALSE) === TRUE) {
      $links_per_page = variable_get('taxonomy_vtn_nodes_links_per_page', 10);
    } 
    else {
      $links_per_page = 0;
    }
    
    // TODO: settings for depth , probably use level of terms ?
    require_once('taxonomy_vtn_overwrites.inc.php');
    $nodes = taxonomy_vtn_select_nodes(array($own_tid), $operator = 'or', $depth = 'all', $use_pager, $order = 'n.sticky DESC', $links_per_page);

    while ($node = db_fetch_object($nodes)) {
      $node = node_load($node->nid);
      // TODO: cut special chars
      $index = drupal_strtoupper(drupal_substr($node->title, 0, 1));
      //print_r($node);
      if ($show_count_comments) {
        if ( $node->comment_count || $show_empty_count_comments) {
          $count_comments = ' ('. format_plural($node->comment_count, '1 comment', '@count comments') .')';
        }
        else {
          $count_comments = '';
        }
      }
      $arr[$index][] = l($node->title, 'node/'. $node->nid) . $count_comments;
      // for <li> style .... $arr[$index][] = array('data' => l($node->title,'node/'.$node->nid) . $count_comments, 'style' => '');
    }

    if ( count($arr) > 0 ) {

      $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');

      // change locale to sort
      if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
        setlocale(LC_ALL, $my_locale);
        ksort($arr, SORT_LOCALE_STRING);
        // set locale back to default
        setlocale(LC_ALL, NULL); // without this count($arr) not work properly
      }
      else {
        ksort($arr); //--- this not support locales
      }

      // if more columns always show index
      if ($count_column_nodes > 1) {
        $show_nodes_index = 1;
        $attr = array();
      }

      $cnt = ceil(count($arr)/$count_column_nodes);
      $k = 0; $col = 0;
      foreach ($arr as $i => $t) {
        if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
          $cell = 'cell'. ++$col;
        }
        if ($show_nodes_index == 0) {
          $i = NULL;
          $attr = array('class' => 'simple-list');
        }
        $$cell .= theme('item_list', $t, $i, 'ul', $attr);
        ++$k;
      }

      // prepare cells and they width
      ($col < $count_column_nodes ) ? $ile = $col : $ile = $count_column_nodes;
      for ($num=1; $num <= $ile; ++$num) {
        $xcell = 'cell'. $num;
        $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
      }
      $row[] = $xcells;

      $output .= '<div class="description">'. $term_obj->description .'</div>';

      $nodes_navigation = '';
      if ($show_back_to_terms) {
        $where_back = t('terms');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
          $where_back = t('master index');
        }
        $nodes_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn/voc/'. $term_obj->vid) .'</span>';
        $output .= '<div class="taxonomy_vtn-navigation">'. $nodes_navigation .'</div>';
      }

      $output .= theme('table', array(), $row, array('class' => 'taxonomy_vtn_nodes', 'id' => 'taxonomy_vtn_nodes'));

      //Cache the fully rendered page in case of aggressive caching
      if (variable_get('taxonomy_vtn_caching', 0) == 1) {
        cache_set('taxonomy_vtn_show_nodes_'. $own_tid, $output);
      }
      $output .= theme('pager');
      return $output;
    }
    else {
      $output  = '<p class="no-data-info">';
      $output .= t('No data to display. Probably this term is empty.');
      $output .= '</p>';
      $nodes_navigation = '';
      if ($show_back_to_terms) {
        $where_back = t('terms');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 0) == 1) {
          $where_back = t('master index');
        }
        $nodes_navigation = '<span class="taxonomy_vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn/voc/'. $term_obj->vid) .'</span>';
        $output .= '<div class="taxonomy_vtn-navigation">'. $nodes_navigation .'</div>';
      }

      //Cache the fully rendered page in case of aggressive caching
      if (variable_get('taxonomy_vtn_caching', 0) == 1) {
        cache_set('taxonomy_vtn_show_nodes_'. $own_tid, $output);
      }

      return $output;
    }
  } // end if cache
}

